"""S3 Sink connector configuration
https://docs.confluent.io/current/connect/kafka-connect-s3
"""

import os
from dataclasses import dataclass

from kafkaconnect.config import ConnectConfig


@dataclass
class S3Config(ConnectConfig):
    """S3 Sink connector configuration"""

    name: str = os.getenv("KAFKA_CONNECT_NAME", "s3-sink")
    """Name of the connector.

    The connector name must be unique accross the cluster.
    """

    connector_class: str = "io.confluent.connect.s3.S3SinkConnector"
    """S3 Sink connector class"""

    format_class: str = "io.confluent.connect.s3.format.parquet.ParquetFormat"
    """The format class to use when writing data to the store."""

    parquet_codec: str = "snappy"
    """The Parquet compression codec to be used for output files."""

    schema_compatibility: str = os.getenv(
        "KAFKA_CONNECT_S3_SCHEMA_COMPATIBILITY", "NONE"
    )
    """The schema compatibility rule.

    The supported configurations are NONE, BACKWARD, FORWARD and FULL.
    """

    s3_bucket_name: str = os.getenv("KAFKA_CONNECT_S3_BUCKET_NAME", "")
    """The S3 Bucket."""

    s3_region: str = os.getenv("KAFKA_CONNECT_S3_REGION", "us-east-1")
    """The AWS region to be used the connector."""

    aws_access_key_id: str = os.getenv("AWS_ACCESS_KEY_ID", "")
    """The AWS access key ID used to authenticate personal AWS credentials."""

    aws_secret_access_key: str = os.getenv("AWS_SECRET_ACCESS_KEY", "")
    """The secret access key used to authenticate personal AWS credentials."""

    topics_dir: str = os.getenv("KAFKA_CONNECT_S3_TOPICS_DIR", "topics")
    """Top level directory to store the data ingested from Kafka."""

    storage_class: str = "io.confluent.connect.s3.storage.S3Storage"
    """The underlying storage layer."""

    flush_size: int = int(os.getenv("KAFKA_CONNECT_S3_FLUSH_SIZE", 3600))
    """Number of records written to store before invoking file commits.

    By default this is set to 6 times the number of records expected for an
    output stream of 1Hz within the default rotate_interval_ms value.
    This way the rotate_interval_ms configuration takes precedence over the
    flush_size configuration. But flush_size still works as a maximum limit
    to invoke file commits when the connect-s3-sink consumer accumulates 3600
    records.
    """

    rotate_interval_ms: int = int(
        os.getenv("KAFKA_CONNECT_S3_ROTATE_INTERVAL_MS", 600000)
    )
    """The time interval in milliseconds to invoke file commits.

    Use this option to control the size of the objects in S3. For example, if
    the output data stream is 1Hz, a rotate interval of 600 seconds will create
    a file with aproximatelly 600 records if less than flush_size. Note that
    the lag of the connect-s3-sink consumer will increase until it accumulates
    records within 600 seconds and it will decrease again after the file
    is commited. For an hourly partitioner this configuration should create 6
    parquet files in the destination path.
    """

    partitioner_class: str = (
        "io.confluent.connect.storage.partitioner.TimeBasedPartitioner"
    )
    """The partitioner to use when writing data to the store."""

    partition_duration_ms: int = int(
        os.getenv("KAFKA_CONNECT_S3_PARTITION_DURATION_MS", 3600000)
    )
    """The duration of a partition in ms, used by the TimeBasedPartitioner.

    The default value is for an hourly partitioner.
    """
    path_format: str = (
        os.getenv(
            "KAFKA_CONNECT_S3_PATH_FORMAT",
            "'year'=YYYY/'month'=MM/'day'=dd/'hour'=HH",
        )
    )
    """Pattern used to format the path in the S3 object name.

    Portion of the path generated by the S3 connector’s partitioner. The
    default is for an hourly partitioner.
    """
    locale: str = os.getenv("KAFKA_CONNECT_S3_LOCALE", "en-US")
    """The locale to use when partitioning with TimeBasedPartitioner."""

    timezone: str = os.getenv("KAFKA_CONNECT_S3_TIMEZONE", "UTC")
    """The timezone to use when partitioning with TimeBasedPartitioner."""

    timestamp_extractor: str = os.getenv(
        "KAFKA_CONNECT_TIMESTAMP_EXTRACTOR", "Record"
    )
    """The extractor determines how to obtain a timestamp from each record.

    Values can be Wallclock to use the system time when
    the record is processed, Record (default) to use the timestamp of the
    Kafka record denoting when it was produced or stored by the broker,
    RecordField to extract the timestamp from one of the fields in the
    record’s value as specified by the timestamp_field configuration property.
    """

    timestamp_field: str = os.getenv("KAFKA_CONNECT_TIMESTAMP_FIELD", "time")
    """The record field to be used as timestamp by the timestamp extractor.

    Only applies if timestamp_extractor is set to `RecordField`.
    """



""" InfluxDB Sink connector configuration
https://docs.lenses.io/connectors/sink/influx.html
"""

import os
from dataclasses import dataclass
from typing import List

from kafkaconnect.config import ConnectConfig


@dataclass
class InfluxConfig(ConnectConfig):
    """InfluxDB connector configuration"""

    name: str = os.getenv("KAFKA_CONNECT_NAME", "influxdb-sink")
    """Name of the connector.

    The connector name must be unique accross the cluster.
    """

    connect_influx_url: str = os.getenv(
        "KAFKA_CONNECT_INFLUXDB_URL", "http://localhost:8086"
    )
    """InfluxDB connection URL."""

    connect_influx_db: str = os.getenv("KAFKA_CONNECT_DATABASE", "")
    """InfluxDB database name."""

    connector_class: str = (
        "com.datamountaineer.streamreactor.connect.influx.InfluxSinkConnector"
    )
    """Stream reactor InfluxDB Sink connector class"""

    connect_influx_kcql: str = ""
    """KCQL queries to extract fields from topics.

    We assume that a topic has a flat structure so that `SELECT * FROM` will
    retrieve all topic fields. This is configuration is derived from the list
    of topics and from the timestamp to use as the InfluxDB time.
    """

    connect_influx_username: str = os.getenv(
        "KAFKA_CONNECT_INFLUXDB_USERNAME", "-"
    )
    """InfluxDB username."""

    connect_influx_password: str = os.getenv(
        "KAFKA_CONNECT_INFLUXDB_PASSWORD", ""
    )
    """InfluxDB password."""

    connect_influx_timestamp: str = os.getenv(
        "KAFKA_CONNECT_INFLUXDB_TIMESTAMP", "sys_time()"
    )
    """Timestamp to use as the InfluxDB time."""

    connect_influx_error_policy: str = os.getenv(
        "KAFKA_CONNECT_ERROR_POLICY", "THROW"
    )
    """Connector error policy configuration.

    See https://docs.lenses.io/connectors/sink/influx.html
    """

    connect_influx_max_retries: str = os.getenv(
        "KAFKA_CONNECT_MAX_RETRIES", "10"
    )
    """Connector error policy configuration.

    See https://docs.lenses.io/connectors/sink/influx.html
    """

    connect_influx_retry_interval: str = os.getenv(
        "KAFKA_CONNECT_RETRY_INTERVAL", "60000"
    )
    """Connector error policy configuration.

    See https://docs.lenses.io/connectors/sink/influx.html
    """

    connect_progress_enabled: bool = os.getenv(
        "KAFKA_CONNECT_PROGRESS_ENABLED", "false"
    ) == "true"
    """Enables the output for how many records have been processed."""

    def update_topics(
        self, topics: List[str], timestamp: str = "sys_time()"
    ) -> None:
        """Update the list of Kafka topics and Influx KCQL queries.

        Parameters
        ----------
        topics : `list`
            List of kafka topics.

        timestamp : `str`
            Timestamp used as influxDB time. Default is ``sys_time()`` you
            can use the name of a field as well.
        """
        # Ensure uniqueness and sort topic names
        topics = list(set(topics))
        topics.sort()
        queries = [
            f"INSERT INTO {t} SELECT * FROM {t} WITHTIMESTAMP {timestamp}"
            for t in topics
        ]
        self.topics = ",".join(topics)
        self.connect_influx_kcql = ";".join(queries)



"""Kafkaconnect application and generic connector configuration."""

__all__ = ["Config", "ConnectConfig"]

import json
import os
from dataclasses import asdict, dataclass
from typing import Any, Dict, List, Tuple


@dataclass
class Config:
    """Kafkaconnect application configuration."""

    broker_url: str = os.getenv("KAFKA_BROKER_URL", "localhost:9092")
    """The Kafka Broker URL.
    """

    connect_url: str = os.getenv("KAFKA_CONNECT_URL", "http://localhost:8083")
    """The Kafka Connect URL.

    The Kafka Connect REST API is used to manage connectors.
    """

    topic_regex: str = os.getenv("KAFKA_CONNECT_TOPIC_REGEX", ".*")
    """Regex to select topics from Kafka."""

    excluded_topics: str = os.getenv("KAFKA_CONNECT_EXCLUDED_TOPICS", "")
    """Comma separated list of topics to exclude from selection."""

    check_interval: int = int(os.getenv("KAFKA_CONNECT_CHECK_INTERVAL", 15000))
    """The interval, in milliseconds, to update the connector.

    Check Kafka for new topics and update the connector configuration.
    """


@dataclass
class ConnectConfig:
    """Generic connector configuration."""

    name: str = ""
    """Name of the connector.

    The connector name must be unique accross the cluster.
    """

    connector_class: str = ""
    """Name of the connector class."""

    topics: str = ""
    """Comma separated list of Kafka topics to read from (sink connectors) or
    to write to (source connectors).
    """

    tasks_max: int = int(os.getenv("KAFKA_CONNECT_TASKS_MAX", "1"))
    """Number of connect tasks to scale out.

    Topics and partitions are distributed accross tasks.
    """

    def update_topics(self, topics: List[str]) -> None:
        """Update the list of Kafka topics.

        Parameters
        ----------
        topics : `list`
            List of kafka topics.
        """
        # Ensure uniqueness and sort topic names
        topics = list(set(topics))
        topics.sort()
        self.topics = ",".join(topics)

    @staticmethod
    def format_field_names(fields: List[Tuple[str, Any]]) -> Dict[str, str]:
        """Dictionary factory to use with the dataclasses.asdict() method.

        Rename the field name replacing '_' by '.' and return a dictionary
        mapping field names to field values.
        """
        result = []
        for f in fields:
            name, value = f
            name = name.replace("_", ".")
            result.append((name, value))
        return dict(result)

    def asjson(self) -> str:
        config = asdict(self, dict_factory=self.format_field_names)
        return json.dumps(config, indent=4, sort_keys=True)
